/* externs.h - Example system declarations (included by every file) */

#define _GNU_SOURCE		// Use the GNU C Library Extensions
#define _FILE_OFFSET_BITS 64	// Select 64-bit version of all file pointers

#define acct acct2
#define index index2

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#undef acct
#undef index

/* Let 'quad' be our 64-bit int type */
#undef quad
#define quad long long

/* Bit values can range from 0 to 63 and can be specified BV0 to BV63 */
enum bitvalues {
#define BV(d) BV##d=(1LL << d)
  BV(0), BV(1), BV(2), BV(3), BV(4), BV(5), BV(6), BV(7), BV(8), BV(9),
  BV(10),BV(11),BV(12),BV(13),BV(14),BV(15),BV(16),BV(17),BV(18),BV(19),
  BV(20),BV(21),BV(22),BV(23),BV(24),BV(25),BV(26),BV(27),BV(28),BV(29),
  BV(30),BV(31),BV(32),BV(33),BV(34),BV(35),BV(36),BV(37),BV(38),BV(39),
  BV(40),BV(41),BV(42),BV(43),BV(44),BV(45),BV(46),BV(47),BV(48),BV(49),
  BV(50),BV(51),BV(52),BV(53),BV(54),BV(55),BV(56),BV(57),BV(58),BV(59),
  BV(60),BV(61),BV(62),BV(63)
#undef BV
};

/* Also allow variable values, BV(0) to BV(63) */
#define BV(d) (1LL << (d))

/* Swap the values of two integers, quadwords, doubles, etc. */
#define XCHG(a,b) (void)({ typeof(a) _temp=a; a=b; b=_temp; })
/* ...The same thing can be done using:  a ^= b, b ^= a, a ^= b */

/* Generic min() and max() functions */
#undef min
#undef max
#define min(x,y) ({ typeof(x) _x=x; typeof(y) _y=y; (_x < _y)?_x:_y; })
#define max(x,y) ({ typeof(x) _x=x; typeof(y) _y=y; (_x > _y)?_x:_y; })

/* Optimal way of keeping a number within a set range */
#define RANGE(x,lo,hi) ({ int _val=x, _lo=lo, _hi=hi; \
                          (_val < _lo)?_lo:(_val > _hi)?_hi:_val; })

/* Determines the number of elements in a static array */
#define NELEM(array) (int)(sizeof(array)/sizeof(array[0]))

/* Count the number of bits set in a word */
#ifdef __x86_64__
# define popcount(x) ({ unsigned long _r=(x), _t; \
			_r -= ((_r >> 1) & 0x5555555555555555); \
			_t  = ((_r >> 2) & 0x3333333333333333); \
			_r  = ((_r & 0x3333333333333333) + _t); \
			_r  = (_r + (_r >> 4)) & 0x0F0F0F0F0F0F0F0F; \
			_r += (_r << 8); \
			_r += (_r << 16); \
			_r += (_r << 32); \
			_r >> 56; })
#else
# define popcount(x) ({ unsigned long _r=(x), _t; \
			_r -= ((_r >> 1) & 0x55555555); \
			_t  = ((_r >> 2) & 0x33333333); \
			_r  = ((_r & 0x33333333) + _t); \
			_r  = (_r + (_r >> 4)) & 0x0F0F0F0F; \
			_r += (_r << 8); \
			_r += (_r << 16); \
			_r >> 24; })
#endif

/* Rotates a 32-bit word right or left */
#define ROR(x,n) (((x) >> ((n) & 0x1F)) | ((x) << (32-((n) & 0x1F))))
#define ROL(x,n) (((x) << ((n) & 0x1F)) | ((x) >> (32-((n) & 0x1F))))

/* Reads the CPU time-stamp counter */
#define rdtsc() ({ unsigned int a, d; \
                   asm volatile("rdtsc" : "=a" (a), "=d" (d)); \
                   (unsigned quad)a | ((unsigned quad)d << 32); })

/* Handle special 64-bit byteswap */
#ifndef htonll
# ifdef __x86_64__
#  define htonll(x) ({ quad _r=x; asm("bswapq %0" : "=r" (_r) : "0" (_r)); _r; })
# else
#  define htonll(x) ({ quad _r=(x), _res; \
		       if(htons(1) == 1) _res=_r; \
		       else _res=(((quad)htonl(_r)) << 32) | htonl(_r >> 32); \
		       _res; })
# endif
# define ntohll(x) htonll(x)
#endif

/* Define our own ctype macros, avoiding locale-based maps */
#undef isspace
#undef isdigit
#undef isalpha
#undef isalnum
#undef isxdigit
#define isspace(c)  ({ char _c=c; (_c == ' ' || _c == '\t'); })
#define isdigit(c)  ({ char _c=c; (_c >= '0' && _c <= '9'); })
#define isalpha(c)  ({ char _c=(c) | 32; (_c >= 'a' && _c <= 'z'); })
#define isalnum(c)  ({ char _ch=c; (isdigit(_ch) || isalpha(_ch)); })
#define isxdigit(c) ({ char _ch=c; (isdigit(_ch) || \
		       ((_ch | 32) >= 'a' && ((_ch | 32) <= 'f'))); })

#undef toupper
#undef tolower
#define toupper(c)  ({ char _c=c; (_c < 'a' || _c > 'z')?_c:(_c & ~32); })
#define tolower(c)  ({ char _c=c; (_c < 'A' || _c > 'Z')?_c:(_c | 32); })

/* Like strncpy but doesn't pad with zeroes. n=buffer size instead of siz-1 */
#define strmaxcpy(d, s, n) ({ char *_d=d; const char *_s=s; \
			      int _n=n, _len=strlen(_s)+1; \
			      if(_n > 0) { \
				memcpy(_d, _s, (_len > _n)?_n:_len); \
				_d[_n-1]='\0'; \
			      } _d; })

/* Returns a pointer to the end null byte in a string */
#define strnul(str)    ({ char *_str=str; _str+strlen(_str); })

/* Copies a string to a pointer allocated on the stack */
#define APTR(ptr, str) ({ char *_str=str; int _len=strlen(_str)+1; \
			  ptr=memcpy(alloca(_len), _str, _len); })

/* Sets a new pointer regardless of previous content; does not free ptr */
#define SPTR(ptr, str) ({ char *_str=str; int _len=strlen(_str)+1; \
			  ptr=memcpy(malloc(_len), _str, _len); })

/* Writes a string to a pointer, freeing previous contents */
#define WPTR(ptr, str) ({ char **_p=&(ptr), *_str=str; \
			  int _len=strlen(_str)+1; \
			  *_p=memcpy((free(*_p),malloc(_len)), _str, _len); })
