/* jump.c - Initializes the jump table at memory address 0x1000 */

#include "sym.h"

/* Include definitions for mmap/munmap() in case it's not listed in our function table */
extern void *mmap();
extern void *munmap();

/* Linux definitions for mmap() */
#define PROT_READ       0x1
#define PROT_WRITE      0x2
#define PROT_EXEC       0x4

#define MAP_PRIVATE     0x02
#define MAP_ANONYMOUS   0x20

#define MAP_FAILED      ((void *)-1)

// Allocates the memory map at address 0x1000 and populates the jump table.
//
void init_jump_table()
{
  unsigned char *s;
  int i;

  /* Allocate room for the jump table */
  s=mmap((void *)0x1000, sizeof(func_list)/sizeof(*func_list)*8, PROT_EXEC|PROT_READ|PROT_WRITE,
         MAP_PRIVATE|MAP_ANONYMOUS, 0, 0);
  if(s == MAP_FAILED)
    return;

  /* Populate the table with 'jmp 32-bit pc-relative' instructions every 8 bytes */
  for(i=0;i < sizeof(func_list)/sizeof(*func_list);i++) {
    *s++=0xe9;
    *(int *)s=func_list[i]-(void *)s-4;
    s+=7;
  }
}

// Removes the memory map at address 0x1000.
//
void free_jump_table()
{
  munmap((void *)0x1000, sizeof(func_list)/sizeof(*func_list)*8);
}
